@@ -55,33 +55,9 @@
 #include <malloc.h>
 #endif
 
-#ifdef OPENCL
-#include <sys/types.h>
-#ifdef __APPLE__
-#include <OpenCL/opencl.h>
-#include <unistd.h>
-#else
-#include <CL/cl.h>
-#ifndef WIN32
-#include <unistd.h>
-#endif
-#endif
-#endif
 
-#ifdef METAL
- #include "mtlpp/mtlpp.hpp"
-#endif
+#define INHOST(_VarName) _VarName
 
-// #ifdef CUDA  // disabling this in the current release, it may be renable later if we find a a way to improve performance
-// #define USE_MINI_KERNELS_CUDA
-// #endif
-
-//////////////////////////////////////////GPU-SPECIFIC
-#if defined(CUDA) || defined(METAL)
-	#define INHOST(_VarName) h ## _VarName
-#else
-	#define INHOST(_VarName) _VarName
-#endif
 
 unsigned int	INHOST(SILENT);
 
@@ -104,7 +80,6 @@ static PyObject *mexFunction(PyObject *self, PyObject *args)
 								INHOST(SensorStart),
 								INHOST(NumberSelRMSPeakMaps),
 								INHOST(NumberSelSensorMaps),
-								INHOST(IndexRMSPeak_ALLV)=0,
 								INHOST(IndexRMSPeak_Vx)=0,
 								INHOST(IndexRMSPeak_Vy)=0,
 								INHOST(IndexRMSPeak_Vz)=0,
@@ -115,7 +90,6 @@ static PyObject *mexFunction(PyObject *self, PyObject *args)
 								INHOST(IndexRMSPeak_Sigmaxz)=0,
 								INHOST(IndexRMSPeak_Sigmayz)=0,
 								INHOST(IndexRMSPeak_Pressure)=0,
-								INHOST(IndexSensor_ALLV)=0,
 								INHOST(IndexSensor_Vx)=0,
 								INHOST(IndexSensor_Vy)=0,
 								INHOST(IndexSensor_Vz)=0,
@@ -348,7 +322,6 @@ static PyObject *mexFunction(PyObject *self, PyObject *args)
 		ERROR_STRING("NumberSelSensorMaps must select at least one type of map to track");
 	//We detect how many maps we need to keep track
 	unsigned int curMapIndex =0;
-	ACCOUNT_RMSPEAK(ALLV);
 	ACCOUNT_RMSPEAK(Vx);
 	ACCOUNT_RMSPEAK(Vy);
 	ACCOUNT_RMSPEAK(Vz);
@@ -367,7 +340,6 @@ static PyObject *mexFunction(PyObject *self, PyObject *args)
 	}
 
 	curMapIndex =0;
-	ACCOUNT_SENSOR(ALLV);
 	ACCOUNT_SENSOR(Vx);
 	ACCOUNT_SENSOR(Vy);
 	ACCOUNT_SENSOR(Vz);
@@ -496,18 +468,7 @@ static PyObject *mexFunction(PyObject *self, PyObject *args)
 
 INHOST(SelK)=INHOST(N3)/2;
 
-#if defined(CUDA) || defined(OPENCL) || defined(METAL)
-  #include "FDTD3D_GPU_VERSION.h"
-#else
-// #ifndef MATLAB_MEX
-//  	Py_BEGIN_ALLOW_THREADS;
-// #endif
-	//////////BEGIN CPU SPECIFC
-  #include "FDTD3D_CPU_VERSION.h"
-//    #ifndef MATLAB_MEX
-//  	Py_END_ALLOW_THREADS;
-//  #endif
-#endif
+#include "FDTD3D_CPU_VERSION.h"
 
    ////END CPU
    time(&end_t);
