@@ -1,6 +1,12 @@
 #ifdef METAL
 #include"kernelparamsMetal.h"
 
+#ifdef METALCOMPUTE
+#define CGID uint
+#else
+#define CGID uint3
+#endif
+#ifndef METALCOMPUTE
 #define METAL_PARAMS\
 	const device unsigned int *p_CONSTANT_BUFFER_UINT [[ buffer(0) ]],\
 	const device mexType * p_CONSTANT_BUFFER_MEX [[ buffer(1) ]],\
@@ -19,11 +25,29 @@
 	device mexType * p_MEX_BUFFER_9 [[ buffer(14) ]],\
 	device mexType * p_MEX_BUFFER_10 [[ buffer(15) ]],\
 	device mexType * p_MEX_BUFFER_11 [[ buffer(16) ]],\
-	uint3 gid[[thread_position_in_grid]])\
-{\
-  _PT i = (_PT) gid.x;\
-  _PT j = (_PT) gid.y;\
-  _PT k = (_PT) gid.z;
+	CGID gid[[thread_position_in_grid]])\
+{
+#else
+#define METAL_PARAMS\
+	const device unsigned int *p_CONSTANT_BUFFER_UINT [[ buffer(0) ]],\
+	const device unsigned int *p_INDEX_MEX [[ buffer(1) ]],\
+	const device unsigned int *p_INDEX_UINT [[ buffer(2) ]],\
+	const device unsigned int *p_UINT_BUFFER [[ buffer(3) ]],\
+	device mexType * p_MEX_BUFFER_0 [[ buffer(4) ]],\
+	device mexType * p_MEX_BUFFER_1 [[ buffer(5) ]],\
+	device mexType * p_MEX_BUFFER_2 [[ buffer(6) ]],\
+	device mexType * p_MEX_BUFFER_3 [[ buffer(7) ]],\
+	device mexType * p_MEX_BUFFER_4 [[ buffer(8) ]],\
+	device mexType * p_MEX_BUFFER_5 [[ buffer(9) ]],\
+	device mexType * p_MEX_BUFFER_6 [[ buffer(10) ]],\
+	device mexType * p_MEX_BUFFER_7 [[ buffer(11) ]],\
+	device mexType * p_MEX_BUFFER_8 [[ buffer(12) ]],\
+	device mexType * p_MEX_BUFFER_9 [[ buffer(13) ]],\
+	device mexType * p_MEX_BUFFER_10 [[ buffer(14) ]],\
+	device mexType * p_MEX_BUFFER_11 [[ buffer(15) ]],\
+	CGID gid[[thread_position_in_grid]])\
+{
+#endif
 #endif
 /// PMLS
 #if defined(METAL)  || defined(USE_MINI_KERNELS_CUDA)
@@ -35,7 +59,9 @@
 #define _ST_PML_6
 #define _PML_KERNEL_CORNER
 #ifdef CUDA
-__global__ void PML_1_StressKernel(InputDataKernel *p,unsigned int nStep, unsigned int TypeSource)
+__global__ void PML_1_StressKernel(
+	#include "kernelparamsOpenCL.h"
+	,unsigned int nStep, unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -53,6 +79,18 @@ __kernel void PML_1_StressKernel(
 #ifdef METAL
 kernel void PML_1_StressKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+  	_PT i = (_PT) gid.x;
+  	_PT j = (_PT) gid.y;
+  	_PT k = (_PT) gid.z;
+	#else
+	#define nN1 (PML_Thickness*2)
+	#define nN2 (PML_Thickness*2)
+	#define nN3 (PML_Thickness*2)
+    _PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
     #include "StressKernel.h" 
 }
@@ -60,7 +98,9 @@ kernel void PML_1_StressKernel(
 
 #define _PML_KERNEL_LEFT_RIGHT
 #ifdef CUDA
-__global__ void PML_2_StressKernel(InputDataKernel *p,unsigned int nStep, unsigned int TypeSource)
+__global__ void PML_2_StressKernel(
+	#include "kernelparamsOpenCL.h"
+	,unsigned int nStep, unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -78,6 +118,18 @@ __kernel void PML_2_StressKernel(
 #ifdef METAL
 kernel void PML_2_StressKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+	_PT i = (_PT) gid.x;
+	_PT j = (_PT) gid.y;
+	_PT k = (_PT) gid.z;
+	#else
+	#define nN1 (PML_Thickness*2)
+	#define nN2 (N2-PML_Thickness*2)
+	#define nN3 (N3-PML_Thickness*2)
+    _PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
     #include "StressKernel.h" 
 }
@@ -85,7 +137,9 @@ kernel void PML_2_StressKernel(
 
 #define _PML_KERNEL_TOP_BOTTOM
 #ifdef CUDA
-__global__ void PML_3_StressKernel(InputDataKernel *p,unsigned int nStep, unsigned int TypeSource)
+__global__ void PML_3_StressKernel(
+	#include "kernelparamsOpenCL.h"
+	,unsigned int nStep, unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -103,6 +157,18 @@ __kernel void PML_3_StressKernel(
 #ifdef METAL
 kernel void PML_3_StressKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+	_PT i = (_PT) gid.x;
+	_PT j = (_PT) gid.y;
+	_PT k = (_PT) gid.z;
+	#else
+	#define nN1 (N1-PML_Thickness*2)
+	#define nN2 (PML_Thickness*2)
+	#define nN3 (N3-PML_Thickness*2)
+	_PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
     #include "StressKernel.h" 
 }
@@ -110,7 +176,9 @@ kernel void PML_3_StressKernel(
 
 #define _PML_KERNEL_FRONT_BACK
 #ifdef CUDA
-__global__ void PML_4_StressKernel(InputDataKernel *p,unsigned int nStep, unsigned int TypeSource)
+__global__ void PML_4_StressKernel(
+	#include "kernelparamsOpenCL.h"
+	,unsigned int nStep, unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -128,6 +196,18 @@ __kernel void PML_4_StressKernel(
 #ifdef METAL
 kernel void PML_4_StressKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+  	_PT i = (_PT) gid.x;
+  	_PT j = (_PT) gid.y;
+  	_PT k = (_PT) gid.z;
+	#else
+	#define nN1 (N1-PML_Thickness*2)
+	#define nN2 (N2-PML_Thickness*2)
+	#define nN3 (PML_Thickness*2)
+	_PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
     #include "StressKernel.h" 
 }
@@ -135,7 +215,9 @@ kernel void PML_4_StressKernel(
 
 #define _PML_KERNEL_LEFT_RIGHT_RODS
 #ifdef CUDA
-__global__ void PML_5_StressKernel(InputDataKernel *p,unsigned int nStep, unsigned int TypeSource)
+__global__ void PML_5_StressKernel(
+	#include "kernelparamsOpenCL.h"
+	,unsigned int nStep, unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -153,6 +235,18 @@ __kernel void PML_5_StressKernel(
 #ifdef METAL
 kernel void PML_5_StressKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+  	_PT i = (_PT) gid.x;
+  	_PT j = (_PT) gid.y;
+  	_PT k = (_PT) gid.z;
+	#else	
+	#define nN1 (PML_Thickness*2)
+	#define nN2 (PML_Thickness*2)
+	#define nN3 (N3-PML_Thickness*2)
+	_PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
     #include "StressKernel.h" 
 }
@@ -160,7 +254,9 @@ kernel void PML_5_StressKernel(
 
 #define _PML_KERNEL_BOTTOM_TOP_RODS
 #ifdef CUDA
-__global__ void PML_6_StressKernel(InputDataKernel *p,unsigned int nStep, unsigned int TypeSource)
+__global__ void PML_6_StressKernel(
+	#include "kernelparamsOpenCL.h"
+	,unsigned int nStep, unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -178,6 +274,18 @@ __kernel void PML_6_StressKernel(
 #ifdef METAL
 kernel void PML_6_StressKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+  	_PT i = (_PT) gid.x;
+  	_PT j = (_PT) gid.y;
+  	_PT k = (_PT) gid.z;
+	#else
+	#define nN1 (N1-PML_Thickness*2)
+	#define nN2 (PML_Thickness*2)
+	#define nN3 (PML_Thickness*2)
+	_PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
     #include "StressKernel.h" 
 }
@@ -205,7 +313,9 @@ kernel void PML_6_StressKernel(
 #define _ST_PML_6
 #endif
 #ifdef CUDA
-__global__ void MAIN_1_StressKernel(InputDataKernel *p,unsigned int nStep, unsigned int TypeSource)
+__global__ void MAIN_1_StressKernel(
+	#include "kernelparamsOpenCL.h"
+	,unsigned int nStep, unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -223,6 +333,18 @@ __kernel void MAIN_1_StressKernel(
 #ifdef METAL
 kernel void MAIN_1_StressKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+  	_PT i = (_PT) gid.x;
+  	_PT j = (_PT) gid.y;
+  	_PT k = (_PT) gid.z;
+	#else	
+	#define nN1 (N1-PML_Thickness*2)
+	#define nN2 (N2-PML_Thickness*2)
+	#define nN3 (N3-PML_Thickness*2)
+	_PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
     #include "StressKernel.h" 
 }
@@ -248,7 +370,9 @@ kernel void MAIN_1_StressKernel(
 #define _PR_PML_3
 #define _PML_KERNEL_CORNER
 #ifdef CUDA
-__global__ void PML_1_ParticleKernel(InputDataKernel *p,unsigned int nStep, unsigned int TypeSource)
+__global__ void PML_1_ParticleKernel(
+	#include "kernelparamsOpenCL.h"
+	,unsigned int nStep, unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -266,6 +390,18 @@ __kernel void PML_1_ParticleKernel(
 #ifdef METAL
 kernel void PML_1_ParticleKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+  	_PT i = (_PT) gid.x;
+  	_PT j = (_PT) gid.y;
+  	_PT k = (_PT) gid.z;
+	#else	
+	#define nN1 (PML_Thickness*2)
+	#define nN2 (PML_Thickness*2)
+	#define nN3 (PML_Thickness*2)
+    _PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
     #include "ParticleKernel.h" 
 }
@@ -273,7 +409,9 @@ kernel void PML_1_ParticleKernel(
 
 #define _PML_KERNEL_LEFT_RIGHT
 #ifdef CUDA
-__global__ void PML_2_ParticleKernel(InputDataKernel *p,unsigned int nStep, unsigned int TypeSource)
+__global__ void PML_2_ParticleKernel(
+	#include "kernelparamsOpenCL.h"
+	,unsigned int nStep, unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -291,6 +429,18 @@ __kernel void PML_2_ParticleKernel(
 #ifdef METAL
 kernel void PML_2_ParticleKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+  	_PT i = (_PT) gid.x;
+  	_PT j = (_PT) gid.y;
+  	_PT k = (_PT) gid.z;
+	#else	
+	#define nN1 (PML_Thickness*2)
+	#define nN2 (N2-PML_Thickness*2)
+	#define nN3 (N3-PML_Thickness*2)
+    _PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
     #include "ParticleKernel.h" 
 }
@@ -298,7 +448,9 @@ kernel void PML_2_ParticleKernel(
 
 #define _PML_KERNEL_TOP_BOTTOM
 #ifdef CUDA
-__global__ void PML_3_ParticleKernel(InputDataKernel *p,unsigned int nStep, unsigned int TypeSource)
+__global__ void PML_3_ParticleKernel(
+	#include "kernelparamsOpenCL.h"
+	,unsigned int nStep, unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -316,6 +468,18 @@ __kernel void PML_3_ParticleKernel(
 #ifdef METAL
 kernel void PML_3_ParticleKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+  	_PT i = (_PT) gid.x;
+  	_PT j = (_PT) gid.y;
+  	_PT k = (_PT) gid.z;
+	#else	
+	#define nN1 (N1-PML_Thickness*2)
+	#define nN2 (PML_Thickness*2)
+	#define nN3 (N3-PML_Thickness*2)
+	_PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
     #include "ParticleKernel.h" 
 }
@@ -323,7 +487,9 @@ kernel void PML_3_ParticleKernel(
 
 #define _PML_KERNEL_FRONT_BACK
 #ifdef CUDA
-__global__ void PML_4_ParticleKernel(InputDataKernel *p,unsigned int nStep, unsigned int TypeSource)
+__global__ void PML_4_ParticleKernel(
+	#include "kernelparamsOpenCL.h"
+	,unsigned int nStep, unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -341,6 +507,18 @@ __kernel void PML_4_ParticleKernel(
 #ifdef METAL
 kernel void PML_4_ParticleKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+  	_PT i = (_PT) gid.x;
+  	_PT j = (_PT) gid.y;
+  	_PT k = (_PT) gid.z;
+	#else	
+	#define nN1 (N1-PML_Thickness*2)
+	#define nN2 (N2-PML_Thickness*2)
+	#define nN3 (PML_Thickness*2)
+	_PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
     #include "ParticleKernel.h" 
 }
@@ -348,7 +526,9 @@ kernel void PML_4_ParticleKernel(
 
 #define _PML_KERNEL_LEFT_RIGHT_RODS
 #ifdef CUDA
-__global__ void PML_5_ParticleKernel(InputDataKernel *p,unsigned int nStep, unsigned int TypeSource)
+__global__ void PML_5_ParticleKernel(
+	#include "kernelparamsOpenCL.h"
+	,unsigned int nStep, unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -366,6 +546,18 @@ __kernel void PML_5_ParticleKernel(
 #ifdef METAL
 kernel void PML_5_ParticleKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+  	_PT i = (_PT) gid.x;
+  	_PT j = (_PT) gid.y;
+  	_PT k = (_PT) gid.z;
+	#else	
+	#define nN1 (PML_Thickness*2)
+	#define nN2 (PML_Thickness*2)
+	#define nN3 (N3-PML_Thickness*2)
+	_PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
     #include "ParticleKernel.h" 
 }
@@ -373,7 +565,9 @@ kernel void PML_5_ParticleKernel(
 
 #define _PML_KERNEL_BOTTOM_TOP_RODS
 #ifdef CUDA
-__global__ void PML_6_ParticleKernel(InputDataKernel *p,unsigned int nStep, unsigned int TypeSource)
+__global__ void PML_6_ParticleKernel(
+	#include "kernelparamsOpenCL.h"
+	,unsigned int nStep, unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -391,6 +585,18 @@ __kernel void PML_6_ParticleKernel(
 #ifdef METAL
 kernel void PML_6_ParticleKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+  	_PT i = (_PT) gid.x;
+  	_PT j = (_PT) gid.y;
+  	_PT k = (_PT) gid.z;
+	#else	
+	#define nN1 (N1-PML_Thickness*2)
+	#define nN2 (PML_Thickness*2)
+	#define nN3 (PML_Thickness*2)
+	_PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
     #include "ParticleKernel.h" 
 }
@@ -411,8 +617,9 @@ kernel void PML_6_ParticleKernel(
 #define _PR_PML_3
 #endif
 #if defined(CUDA)
-__global__ void MAIN_1_ParticleKernel(InputDataKernel * p,
-			unsigned int nStep,unsigned int TypeSource)
+__global__ void MAIN_1_ParticleKernel(
+			#include "kernelparamsOpenCL.h"
+			,unsigned int nStep,unsigned int TypeSource)
 {
 	_PT i = (_PT) (blockIdx.x * blockDim.x + threadIdx.x);
     _PT j = (_PT) (blockIdx.y * blockDim.y + threadIdx.y);
@@ -431,6 +638,18 @@ __kernel void MAIN_1_ParticleKernel(
 #ifdef METAL
 kernel void MAIN_1_ParticleKernel(
 	METAL_PARAMS
+	#ifndef METALCOMPUTE
+  	_PT i = (_PT) gid.x;
+  	_PT j = (_PT) gid.y;
+  	_PT k = (_PT) gid.z;
+	#else
+	#define nN1 (N1-PML_Thickness*2)
+	#define nN2 (N2-PML_Thickness*2)
+	#define nN3 (N3-PML_Thickness*2)
+	_PT k = (_PT) (gid/(nN1*nN2));
+  	_PT j = (_PT) ((gid - k*nN1*nN2)/nN1);
+  	_PT i = (_PT) (gid - k*nN1*nN2-j*nN1);
+	#endif
 #endif
 	#include "ParticleKernel.h"
 }
@@ -489,21 +708,23 @@ kernel void SnapShot(
 
     if (i>=N1 || j >=N2)
 		return;
-	mexType accum=0.0;
-	for (unsigned int CurZone=0;CurZone<ZoneCount;CurZone++)
-		{
-			_PT index=Ind_Sigma_xx(i,j,(_PT)SelK);
-			accum+=(Sigma_xx_pr[index]+Sigma_yy_pr[index]+Sigma_zz_pr[index])/3.0;
+	// mexType accum=0.0;
+	// for (unsigned int CurZone=0;CurZone<ZoneCount;CurZone++)
+	// 	{
+	// 		_PT index=Ind_Sigma_xx(i,j,(_PT)SelK);
+	// 		accum+=(Sigma_xx_pr[index]+Sigma_yy_pr[index]+Sigma_zz_pr[index])/3.0;
 
-		}
+	// 	}
 
-		Snapshots_pr[IndN1N2Snap(i,j)+CurrSnap*N1*N2]=accum/ZoneCount;
+	// 	Snapshots_pr[IndN1N2Snap(i,j)+CurrSnap*N1*N2]=accum/ZoneCount;
 }
 
 #if defined(CUDA)
-__global__ void SensorsKernel(InputDataKernel * p,
-													  unsigned int * IndexSensorMap_pr,
-														unsigned int nStep)
+__global__ void SensorsKernel(
+	#include "kernelparamsOpenCL.h"
+	,mexType * SensorOutput_pr,
+	unsigned int * IndexSensorMap_pr,
+	unsigned int nStep)
 {
 	unsigned int sj =blockIdx.x * blockDim.x + threadIdx.x;
 #endif
@@ -520,6 +741,7 @@ __kernel void SensorsKernel(
 
 #define IndexSensorMap_pr k_IndexSensorMap_pr
 
+#ifndef METALCOMPUTE
 kernel void SensorsKernel(
 	const device unsigned int *p_CONSTANT_BUFFER_UINT [[ buffer(0) ]],
 	const device mexType * p_CONSTANT_BUFFER_MEX [[ buffer(1) ]],
@@ -539,6 +761,26 @@ kernel void SensorsKernel(
 	device mexType * p_MEX_BUFFER_10 [[ buffer(15) ]],
 	device mexType * p_MEX_BUFFER_11 [[ buffer(16) ]],
 	uint gid[[thread_position_in_grid]])
+#else
+kernel void SensorsKernel(
+	const device unsigned int *p_CONSTANT_BUFFER_UINT [[ buffer(0) ]],
+	const device unsigned int *p_INDEX_MEX [[ buffer(1) ]],
+	const device unsigned int *p_INDEX_UINT [[ buffer(2) ]],
+	const device unsigned int *p_UINT_BUFFER [[ buffer(3) ]],
+	device mexType * p_MEX_BUFFER_0 [[ buffer(4) ]],
+	device mexType * p_MEX_BUFFER_1 [[ buffer(5) ]],
+	device mexType * p_MEX_BUFFER_2 [[ buffer(6) ]],
+	device mexType * p_MEX_BUFFER_3 [[ buffer(7) ]],
+	device mexType * p_MEX_BUFFER_4 [[ buffer(8) ]],
+	device mexType * p_MEX_BUFFER_5 [[ buffer(9) ]],
+	device mexType * p_MEX_BUFFER_6 [[ buffer(10) ]],
+	device mexType * p_MEX_BUFFER_7 [[ buffer(11) ]],
+	device mexType * p_MEX_BUFFER_8 [[ buffer(12) ]],
+	device mexType * p_MEX_BUFFER_9 [[ buffer(13) ]],
+	device mexType * p_MEX_BUFFER_10 [[ buffer(14) ]],
+	device mexType * p_MEX_BUFFER_11 [[ buffer(15) ]],
+	uint gid[[thread_position_in_grid]])
+#endif
 {
 	_PT sj = (_PT) gid;
 #endif
