@@ -4,16 +4,16 @@
 import PackageDescription
 
 let package = Package(
-    name: "RayleighMetal",
+    name: "BabelMetal",
     platforms: [
         .macOS("11")
     ],
     products: [
         // Products define the executables and libraries produced by a package, and make them visible to other packages.
         .library(
-            name: "RayleighMetal",
+            name: "BabelMetal",
             type: .dynamic,
-            targets: ["RayleighMetal"]),
+            targets: ["BabelMetal"]),
     ],
     dependencies: [
         // Dependencies declare other packages that this package depends on.
@@ -23,7 +23,7 @@ let package = Package(
         // Targets are the basic building blocks of a package. A target can define a module or a test suite.
         // Targets can depend on other targets in this package, and on products in packages which this package depends on.
         .target(
-            name: "RayleighMetal",
+            name: "BabelMetal",
             dependencies: []),
     ]
 )
