@@ -202,69 +202,6 @@ int mxcheck(int result, char const *const func, const char *const file, int cons
 
 
 #if defined(CUDA)
-//declaration of constant values for FDTD solution
-__constant__ mexType DT;
-__constant__ unsigned int N1;
-__constant__ unsigned int N2;
-__constant__ unsigned int N3;
-__constant__ unsigned int Limit_I_low_PML;
-__constant__ unsigned int Limit_J_low_PML;
-__constant__ unsigned int Limit_K_low_PML;
-__constant__ unsigned int Limit_I_up_PML;
-__constant__ unsigned int Limit_J_up_PML;
-__constant__ unsigned int Limit_K_up_PML;
-__constant__ unsigned int SizeCorrI;
-__constant__ unsigned int SizeCorrJ;
-__constant__ unsigned int SizeCorrK;
-__constant__ unsigned int PML_Thickness;
-__constant__ unsigned int NumberSources;
-__constant__ unsigned int LengthSource;
-__constant__ unsigned int NumberSensors;
-__constant__ unsigned int TimeSteps;
-
-__constant__ unsigned int SizePML;
-__constant__ unsigned int SizePMLxp1;
-__constant__ unsigned int SizePMLyp1;
-__constant__ unsigned int SizePMLzp1;
-__constant__ unsigned int SizePMLxp1yp1zp1;
-__constant__ unsigned int ZoneCount;
-
-__constant__ unsigned int SelRMSorPeak;
-__constant__ unsigned int SelMapsRMSPeak;
-__constant__ _PT IndexRMSPeak_ALLV;
-__constant__ _PT IndexRMSPeak_Vx;
-__constant__ _PT IndexRMSPeak_Vy;
-__constant__ _PT IndexRMSPeak_Vz;
-__constant__ _PT IndexRMSPeak_Sigmaxx;
-__constant__ _PT IndexRMSPeak_Sigmayy;
-__constant__ _PT IndexRMSPeak_Sigmazz;
-__constant__ _PT IndexRMSPeak_Sigmaxy;
-__constant__ _PT IndexRMSPeak_Sigmaxz;
-__constant__ _PT IndexRMSPeak_Sigmayz;
-__constant__ _PT IndexRMSPeak_Pressure;
-__constant__ _PT NumberSelRMSPeakMaps;
-
-__constant__ unsigned int SelMapsSensors;
-__constant__ _PT IndexSensor_ALLV;
-__constant__ _PT IndexSensor_Vx;
-__constant__ _PT IndexSensor_Vy;
-__constant__ _PT IndexSensor_Vz;
-__constant__ _PT IndexSensor_Sigmaxx;
-__constant__ _PT IndexSensor_Sigmayy;
-__constant__ _PT IndexSensor_Sigmazz;
-__constant__ _PT IndexSensor_Sigmaxy;
-__constant__ _PT IndexSensor_Sigmaxz;
-__constant__ _PT IndexSensor_Sigmayz;
-__constant__ _PT IndexSensor_Pressure;
-__constant__ _PT NumberSelSensorMaps;
-__constant__ _PT SensorSubSampling;
-__constant__ _PT SensorStart;
-
-__constant__ mexType gpuInvDXDTpluspr[MAX_SIZE_PML];
-__constant__ mexType gpuDXDTminuspr[MAX_SIZE_PML];
-__constant__ mexType gpuInvDXDTplushppr[MAX_SIZE_PML];
-__constant__ mexType gpuDXDTminushppr[MAX_SIZE_PML];
-
 //Calculate the block dimensions
 #define CUDA_GRID_BLOC_BASE(__KERNEL__)\
   dim3 dimBlock## __KERNEL__;  \
@@ -661,31 +598,31 @@ char* load_file(char const* path)
 #endif
 
 #ifdef METAL
- 
-#define GET_KERNEL_STRESS_FUNCTION(__ID__)\
-    mtlpp::Function __ID__ ##_StressKernelFunc = library.NewFunction(#__ID__ "_StressKernel");\
-    mxcheckGPUErrors(((int)__ID__ ##_StressKernelFunc));\
-    mtlpp::ComputePipelineState __ID__ ##_computePipelineStateStress = device.NewComputePipelineState(__ID__ ##_StressKernelFunc, nullptr);\
-    mxcheckGPUErrors(((int)__ID__ ##_computePipelineStateStress));
-
-#define GET_KERNEL_PARTICLE_FUNCTION(__ID__)\
-    mtlpp::Function __ID__ ##_ParticleKernelFunc = library.NewFunction(#__ID__ "_ParticleKernel");\
-    mxcheckGPUErrors(((int)__ID__ ##_ParticleKernelFunc));\
-    mtlpp::ComputePipelineState __ID__ ##_computePipelineStateParticle = device.NewComputePipelineState(__ID__ ##_ParticleKernelFunc, nullptr);\
-    mxcheckGPUErrors(((int)__ID__ ##_computePipelineStateParticle));
-
+extern int SymbolInitiation_uint(unsigned int, unsigned int);
+extern int SymbolInitiation_mex(unsigned int, mexType);
+extern int CompleteCopyMEX(int, mexType *, unsigned int, unsigned int);
+extern int CompleteCopyUInt(int, unsigned int *, unsigned int);
+extern unsigned int GetThreadExecutionWidth(char[], int);
+extern unsigned int GetMaxTotalThreadsPerThreadgroup(char[], int);
+extern void EncodeStress(const char[], unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int,unsigned int, unsigned int, unsigned int);
+extern void EncodeParticle(const char[], unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int,unsigned int, unsigned int, unsigned int);
+extern float* CopyFromGPUMEX(unsigned long);
+extern unsigned int* CopyFromGPUUInt();
+extern float* CopyFromGpuSnapshot();
+extern void CreateAndCopyFromMXVarOnGPUSnapShot(int, mexType *);
 
 #define SET_USER_LOCAL_STRESS(__ID__)\
       __ID__ ##_local_stress[0]=(size_t)ManualLocalSize_pr[0];\
       __ID__ ##_local_stress[1]=(size_t)ManualLocalSize_pr[1];\
       __ID__ ##_local_stress[2]=(size_t)ManualLocalSize_pr[2];
 
+
 #define CALC_USER_LOCAL_STRESS(__ID__)\
 {\
-      unsigned int w = __ID__ ##_computePipelineStateStress.GetThreadExecutionWidth();\
-      unsigned int h = __ID__ ##_computePipelineStateStress.GetMaxTotalThreadsPerThreadgroup() / w;\
+      unsigned int w = GetThreadExecutionWidth(#__ID__, 0);\
+      unsigned int h = GetMaxTotalThreadsPerThreadgroup(#__ID__, 0) / w;\
       unsigned int z =1;\
-      if (h%2==0)\
+	  if (h%2==0)\
       {\
         h=h/2;\
         z=2;\
@@ -703,8 +640,8 @@ char* load_file(char const* path)
 
 #define CALC_USER_LOCAL_PARTICLE(__ID__)\
 {\
-      unsigned int w = __ID__ ##_computePipelineStateParticle.GetThreadExecutionWidth();\
-      unsigned int h = __ID__ ##_computePipelineStateParticle.GetMaxTotalThreadsPerThreadgroup() / w;\
+      unsigned int w = GetThreadExecutionWidth(#__ID__, 1);\
+      unsigned int h = GetMaxTotalThreadsPerThreadgroup(#__ID__, 1) / w;\
       unsigned int z =1;\
       if (h%2==0)\
       {\
@@ -777,49 +714,11 @@ char* load_file(char const* path)
   PRINTF("PML_6_global_" #__TYPE__ "=[%i,%i,%i],[%i,%i,%i]\n",PML_6_global_## __TYPE__[0],PML_6_global_## __TYPE__[1],PML_6_global_## __TYPE__[2],\
     PML_6_global_## __TYPE__[0]*PML_6_local_## __TYPE__[0],PML_6_global_## __TYPE__[1]*PML_6_local_## __TYPE__[1],PML_6_global_## __TYPE__[2]*PML_6_local_## __TYPE__[2]);
  
-  #define ENCODE_STRESS(__ID__)\
-        mtlpp::ComputeCommandEncoder __ID__ ##StressEncoder = StresscommandBuffer.ComputeCommandEncoder();\
-        mxcheckGPUErrors(((int)__ID__ ##StressEncoder));\
-        __ID__ ##StressEncoder.SetBuffer(_CONSTANT_BUFFER_UINT, 0, 0);\
-        __ID__ ##StressEncoder.SetBuffer(_CONSTANT_BUFFER_MEX, 0, 1);\
-        __ID__ ##StressEncoder.SetBuffer(_INDEX_MEX, 0, 2);\
-        __ID__ ##StressEncoder.SetBuffer(_INDEX_UINT, 0, 3);\
-        __ID__ ##StressEncoder.SetBuffer(_UINT_BUFFER, 0, 4);\
-        for (_PT ii=0;ii<12;ii++)\
-            __ID__ ##StressEncoder.SetBuffer(_MEX_BUFFER[ii], 0, 5+ii);\
-        __ID__ ##StressEncoder.SetComputePipelineState(__ID__ ##_computePipelineStateStress);\
-        __ID__ ##StressEncoder.DispatchThreadgroups(\
-            mtlpp::Size(\
-              __ID__ ##_global_stress[0],\
-              __ID__ ##_global_stress[1],\
-              __ID__ ##_global_stress[2]),\
-            mtlpp::Size(\
-              __ID__ ##_local_stress[0],\
-              __ID__ ##_local_stress[1],\
-              __ID__ ##_local_stress[2]));\
-        __ID__ ##StressEncoder.EndEncoding();
+  #define ENCODE_STRESS(__ID__,_Gx,_Gy,_Gz)\
+		EncodeStress(#__ID__, __ID__ ##_global_stress[0], __ID__ ##_global_stress[1], __ID__ ##_global_stress[2], __ID__ ##_local_stress[0], __ID__ ##_local_stress[1], __ID__ ##_local_stress[2],_Gx,_Gy,_Gz);
 
-#define ENCODE_PARTICLE(__ID__)\
-        mtlpp::ComputeCommandEncoder __ID__ ##ParticleEncoder = StresscommandBuffer.ComputeCommandEncoder();\
-        mxcheckGPUErrors(((int)__ID__ ##ParticleEncoder));\
-        __ID__ ##ParticleEncoder.SetBuffer(_CONSTANT_BUFFER_UINT, 0, 0);\
-        __ID__ ##ParticleEncoder.SetBuffer(_CONSTANT_BUFFER_MEX, 0, 1);\
-        __ID__ ##ParticleEncoder.SetBuffer(_INDEX_MEX, 0, 2);\
-        __ID__ ##ParticleEncoder.SetBuffer(_INDEX_UINT, 0, 3);\
-        __ID__ ##ParticleEncoder.SetBuffer(_UINT_BUFFER, 0, 4);\
-        for (_PT ii=0;ii<12;ii++)\
-            __ID__ ##ParticleEncoder.SetBuffer(_MEX_BUFFER[ii], 0, 5+ii);\
-        __ID__ ##ParticleEncoder.SetComputePipelineState(__ID__ ##_computePipelineStateParticle);\
-        __ID__ ##ParticleEncoder.DispatchThreadgroups(\
-            mtlpp::Size(\
-              __ID__ ##_global_particle[0],\
-              __ID__ ##_global_particle[1],\
-              __ID__ ##_global_particle[2]),\
-            mtlpp::Size(\
-              __ID__ ##_local_particle[0],\
-              __ID__ ##_local_particle[1],\
-              __ID__ ##_local_particle[2]));\
-        __ID__ ##ParticleEncoder.EndEncoding();
+#define ENCODE_PARTICLE(__ID__,_Gx,_Gy,_Gz)\
+		EncodeParticle(#__ID__, __ID__ ##_global_particle[0], __ID__ ##_global_particle[1], __ID__ ##_global_particle[2], __ID__ ##_local_particle[0], __ID__ ##_local_particle[1], __ID__ ##_local_particle[2],_Gx,_Gy,_Gz);
 
 
 #define mxcheckGPUErrors(val)           mxcheck ( (val), #val, __FILE__, __LINE__ )
@@ -827,36 +726,30 @@ char* load_file(char const* path)
 
 #define InitSymbol(_NameVar,_datatype,_gtype)\
 {\
-	if (_gtype==G_INT)\
-	{\
-			_datatype * inData = static_cast<_datatype *>(_CONSTANT_BUFFER_UINT.GetContents());\
-			inData[CInd_ ## _NameVar] = INHOST(_NameVar);\
-			_CONSTANT_BUFFER_UINT.DidModify(ns::Range((CInd_ ## _NameVar)*sizeof(_datatype), sizeof(_datatype)));\
+	if(_gtype==G_INT)\
+{\
+			if(SymbolInitiation_uint(CInd_ ## _NameVar, INHOST(_NameVar)) != 0){\
+				ERROR_STRING("Something went wrong in symbol initiation.")\
+			}\
 	}\
 	else\
 	{\
-	 	_datatype * inData = static_cast<_datatype *>(_CONSTANT_BUFFER_MEX.GetContents());\
-		inData[CInd_ ## _NameVar] = INHOST(_NameVar);\
-		_CONSTANT_BUFFER_MEX.DidModify(ns::Range((CInd_ ## _NameVar)*sizeof(_datatype), sizeof(_datatype)));\
+				if(SymbolInitiation_mex(CInd_ ## _NameVar, INHOST(_NameVar)) != 0){\
+				ERROR_STRING("Something went wrong in symbol initiation.")\
+			}\
 	}\
 }
 
 #define InitSymbolArray(_NameVar,_gtype,__Limit)\
-if (_gtype==G_INT)\
-{\
-		unsigned int * inData = static_cast<unsigned int *>(_CONSTANT_BUFFER_UINT.GetContents());\
-		for (unsigned int _n=0;_n<__Limit;_n++)\
-		{\
-			inData[_n + CInd_ ## _NameVar] = _NameVar ## _pr[_n];\
-		}\
-}\
+if (_gtype==G_INT){\		
+for (unsigned int _n=0; _n<__Limit;_n++){\
+			SymbolInitiation_uint((_n + CInd_ ## _NameVar), (_NameVar ## _pr[_n]));\
+}}\
 else\
 {\
-	mexType * inData = static_cast<mexType *>(_CONSTANT_BUFFER_MEX.GetContents());\
-	for (unsigned int _n=0;_n<__Limit;_n++)\
-	{\
-		inData[_n + CInd_ ## _NameVar] = _NameVar ## _pr[_n];\
-	}\
+		for (unsigned int _n=0; _n<__Limit;_n++){\
+			SymbolInitiation_mex((_n + CInd_ ## _NameVar), _NameVar ## _pr[_n]);\
+		}\ 
 }\
 
 const _PT  _IndexDataMetal(const char * NameVar)
@@ -966,56 +859,41 @@ const _PT  _IndexDataMetal(const char * NameVar)
 
 #define CreateAndCopyFromMXVarOnGPU2(_NameVar,_dataType) SizeCopy =GET_NUMBER_ELEMS(_NameVar); \
 					 PRINTF("Allocating in GPU for " #_NameVar " %lu elem.\n",(_PT)SizeCopy);\
-					 gpu_ ## _NameVar ##_pr = device.NewBuffer(sizeof(_dataType) * \
-				              SizeCopy,\
-				             mtlpp::ResourceOptions::StorageModeManaged);\
-				   mxcheckGPUErrors(((int)gpu_ ## _NameVar ##_pr));\
-					 {\
-					      _dataType * inData = static_cast<_dataType*>(gpu_ ## _NameVar ##_pr.GetContents());\
-					      memcpy(inData,_NameVar ## _pr ,sizeof(_dataType) * SizeCopy);\
-					      gpu_ ## _NameVar ##_pr.DidModify(ns::Range( 0, sizeof(_dataType) *SizeCopy));\
-					  }
+					 CreateAndCopyFromMXVarOnGPUSnapShot(SizeCopy, _NameVar ## _pr);\
+
 
 #define CopyFromGPUToMX(_NameVar,_dataType) 	 SizeCopy = GET_NUMBER_ELEMS(_NameVar ##_res)*INHOST(ZoneCount); \
 		if (NULL!=strstr(#_dataType,"mexType"))\
 	 {	\
 	 	 _PT subArray = _IndexDataMetal(#_NameVar);\
-		 _dataType * inData = static_cast<_dataType*>(_MEX_BUFFER[subArray].GetContents());\
+		 _dataType * inData = (_dataType *)CopyFromGPUMEX(subArray);\
 		 memcpy(_NameVar ## _pr,&inData[HOST_INDEX_MEX[CInd_ ##_NameVar][0]],sizeof(_dataType) *SizeCopy );\
 	 } \
 	 else\
 	 {\
-		 _dataType * inData = static_cast<_dataType*>(_UINT_BUFFER.GetContents());\
+		 _dataType * inData = (_dataType *)CopyFromGPUUInt();\
 		 memcpy(_NameVar ## _pr,&inData[HOST_INDEX_UINT[CInd_ ##_NameVar][0]],sizeof(_dataType) *SizeCopy );\
 	 }
+		
 
 #define CopyFromGPUToMX3(_NameVar,_dataType) 	 SizeCopy = GET_NUMBER_ELEMS(_NameVar); \
-		if (NULL!=strstr(#_dataType,"mexType"))\
-	 {	\
-		 _dataType * inData = static_cast<_dataType*>(gpu_ ## _NameVar ## _pr.GetContents());\
+		 _dataType * inData = (_dataType *)CopyFromGpuSnapshot();\
 		 memcpy(_NameVar ## _pr,inData,sizeof(_dataType) *SizeCopy );\
-	 } \
-	 else\
-	 {\
-		 _dataType * inData = static_cast<_dataType*>(gpu_ ## _NameVar ## _pr.GetContents());\
-		 memcpy(_NameVar ## _pr,inData,sizeof(_dataType) *SizeCopy );\
-	 }
-
 
  #define CopyFromGPUToMX4(_NameVar,_dataType) 	 SizeCopy = GET_NUMBER_ELEMS(_NameVar); \
 	 		if (NULL!=strstr(#_dataType,"mexType"))\
-	 	 {	\
-		     _PT subArray = _IndexDataMetal(#_NameVar);\
-	 		 _dataType * inData = static_cast<_dataType*>(_MEX_BUFFER[subArray].GetContents());\
-			 memcpy(_NameVar ## _pr,&inData[HOST_INDEX_MEX[CInd_ ##_NameVar][0]],sizeof(_dataType) *SizeCopy );\
-			  } \
+		{\
+			_PT subArray = _IndexDataMetal(#_NameVar);\
+	 		_dataType * inData = (_dataType *)(CopyFromGPUMEX(subArray));\
+			memcpy(_NameVar ## _pr,&inData[HOST_INDEX_MEX[CInd_##_NameVar][0]],sizeof(_dataType) *SizeCopy );\
+		}\
 	 	 else\
 	 	 {\
-	 		 _dataType * inData = static_cast<_dataType*>(_UINT_BUFFER.GetContents());\
-			 memcpy(_NameVar ## _pr,&inData[HOST_INDEX_UINT[CInd_ ##_NameVar][0]],sizeof(_dataType) *SizeCopy );\
-			 }
+	 		_dataType * inData = (_dataType *)(CopyFromGPUUInt());\
+			memcpy(_NameVar ## _pr,&inData[HOST_INDEX_UINT[CInd_##_NameVar][0]],sizeof(_dataType) *SizeCopy );\
+			}
 
-	 // METAL is c++ based and their own clasess release the memory
+	 // Memory Freedom is incorporated into freeGPUextern() in FDTD3D_GPU_VERSION
 
 		#define ownGPUFree(_NameVar) { }
 
@@ -1023,16 +901,16 @@ const _PT  _IndexDataMetal(const char * NameVar)
 
 		#define CompleteCopyToGpu(_NameVar,_dataType) 	 SizeCopy = GET_NUMBER_ELEMS(_NameVar); \
 		if (NULL!=strstr(#_dataType,"mexType"))\
-	 {	\
-	 	_PT subArray = _IndexDataMetal(#_NameVar);\
-		 _dataType * inData = static_cast<_dataType*>(_MEX_BUFFER[subArray].GetContents());\
-		 memcpy(&inData[HOST_INDEX_MEX[CInd_ ##_NameVar][0]],_NameVar ## _pr,sizeof(_dataType) *SizeCopy );\
-	 } \
-	 else\
-	 {\
-		 _dataType * inData = static_cast<_dataType*>(_UINT_BUFFER.GetContents());\
-		 memcpy(&inData[HOST_INDEX_UINT[CInd_ ##_NameVar][0]],_NameVar ## _pr,sizeof(_dataType) *SizeCopy );\
-	 	 }
+		{\
+			_PT subArray = _IndexDataMetal(#_NameVar);\
+			mexType * dummy = (mexType*)_NameVar ## _pr;\
+			CompleteCopyMEX(SizeCopy, dummy, HOST_INDEX_MEX[CInd_ ##_NameVar][0], subArray);\
+		}\
+	 	else\
+	 	{\
+		 	unsigned int * dummy = (unsigned int*)_NameVar ## _pr;\
+			CompleteCopyUInt(SizeCopy, dummy, HOST_INDEX_UINT[CInd_ ##_NameVar][0]);\
+		}
 
 #endif
 
